@@ -66,7 +66,8 @@ function get_bloginfo() {}
  * @ignore
  */
 function is_admin() {
-	return true;}
+	return true;
+}
 
 /**
  * @ignore
@@ -93,18 +94,9 @@ function includes_url() {}
  */
 function wp_guess_url() {}
 
-if ( ! function_exists( 'json_encode' ) ) :
-	/**
-	 * @ignore
-	 */
-	function json_encode() {}
-endif;
-
 function get_file( $path ) {
 
-	if ( function_exists( 'realpath' ) ) {
-		$path = realpath( $path );
-	}
+	$path = realpath( $path );
 
 	if ( ! $path || ! @is_file( $path ) ) {
 		return '';
